﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.Owin;
using Microsoft.AspNet.Identity.EntityFramework;
using Owin;

using ASPNETWebFormsIdentity.Models;

namespace ASPNETWebFormsIdentity.Administrator
{
    public partial class Default : System.Web.UI.Page
    {
        /// <summary>
        /// ページの読み込み時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_Load(object sender, EventArgs e)
        {
            //PostBack時以外
            if (!IsPostBack)
            {
                var manager = Context.GetOwinContext().GetUserManager<ApplicationUserManager>();

                ddlRole.Items.Clear();
                ddlUser.Items.Clear();

                foreach (var user in manager.Users)
                {
                    ddlUser.Items.Add(new ListItem(user.UserName, user.UserName));
                }

                foreach (var role in manager.Roles)
                {
                    ddlRole.Items.Add(new ListItem(role.Name, role.Name));
                }

                ddlRole.SelectedIndex = 0;
                dispSelRoleUser();
            }
        }

        /// <summary>
        /// ロールの追加
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnAddRole_Click(object sender, EventArgs e)
        {
            if (txtTargetUser.Text.Length == 0)
            {
                return;
            }

            var manager = Context.GetOwinContext().GetUserManager<ApplicationUserManager>();
            
            string selRole = ddlRole.SelectedValue;
            ApplicationUser user = manager.FindByName(txtTargetUser.Text);

            manager.AddToRole(user.Id, selRole);

            dispSelRoleUser();
        }

        /// <summary>
        /// 選択されたロールのユーザ一覧を表示
        /// </summary>
        private void dispSelRoleUser()
        {
            string selRole = ddlRole.SelectedValue;

            if (selRole.Length == 0)
            {
                return;
            }

            var manager = Context.GetOwinContext().GetUserManager<ApplicationUserManager>();

            IdentityRole role = manager.RoleByName(selRole);
            lstRoleUser.Items.Clear();

            if (role.Users == null)
            {
                return;
            }

            foreach (var roleUser in role.Users)
            {
                string userName = manager.FindById(roleUser.UserId).UserName;
                lstRoleUser.Items.Add(new ListItem(userName, userName));
            }
        }

        /// <summary>
        /// ロール選択変更時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ddlRole_SelectedIndexChanged(object sender, EventArgs e)
        {
            dispSelRoleUser();
        }

        /// <summary>
        /// ユーザの選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnToTarget_Click(object sender, EventArgs e)
        {
            string selUser = ddlUser.SelectedValue;
            txtTargetUser.Text = selUser;
        }

        /// <summary>
        /// ユーザ削除
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnUserDel_Click(object sender, EventArgs e)
        {
            var manager = Context.GetOwinContext().GetUserManager<ApplicationUserManager>();

            string selUser = ddlUser.SelectedValue;
            ApplicationUser user = manager.FindByName(selUser);

            manager.Delete(user);
            ddlUser.Items.Clear();

            foreach (var userItem in manager.Users)
            {
                ddlUser.Items.Add(new ListItem(userItem.UserName, userItem.UserName));
            }

            dispSelRoleUser();
        }

        /// <summary>
        /// ユーザ削除
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnDelRole_Click(object sender, EventArgs e)
        {
            if (txtTargetUser.Text.Length == 0)
            {
                return;
            }

            var manager = Context.GetOwinContext().GetUserManager<ApplicationUserManager>();

            string selRole = ddlRole.SelectedValue;
            ApplicationUser user = manager.FindByName(txtTargetUser.Text);

            manager.RemoveFromRole(user.Id, selRole);

            dispSelRoleUser();
        }

        /// <summary>
        /// 新規ロール追加
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnNewRole_Click(object sender, EventArgs e)
        {
            var manager = Context.GetOwinContext().GetUserManager<ApplicationUserManager>();
            manager.CreateRole(txtNewRole.Text);

            ddlRole.Items.Clear();
        
            foreach (var role in manager.Roles)
            {
                ddlRole.Items.Add(new ListItem(role.Name, role.Name));
            }

            ddlRole.SelectedIndex = 0;
            dispSelRoleUser();
        }

    }
}