﻿using System.Threading.Tasks;
using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.EntityFramework;
using Microsoft.AspNet.Identity.Owin;
using Microsoft.Owin;
using ASPNETWebFormsIdentity.Models;

using System.Collections.Generic;
using System.Linq;

using System;

namespace ASPNETWebFormsIdentity
{
    // このアプリケーションで使用されるアプリケーション ユーザー マネージャーを設定します。UserManager は ASP.NET Identity の中で定義されており、このアプリケーションで使用されます。
    public class ApplicationUserManager : UserManager<ApplicationUser>
    {
        public ApplicationUserManager(IUserStore<ApplicationUser> store)
            : base(store)
        {
        }

        public static ApplicationUserManager Create(IdentityFactoryOptions<ApplicationUserManager> options, IOwinContext context)
        {
            var manager = new ApplicationUserManager(new UserStore<ApplicationUser>(context.Get<ApplicationDbContext>()));
            // ユーザー名の検証ロジックを設定します
            manager.UserValidator = new UserValidator<ApplicationUser>(manager)
            {
                AllowOnlyAlphanumericUserNames = false,
                RequireUniqueEmail = true
            };
            // パスワードの検証ロジックを設定します
            manager.PasswordValidator = new PasswordValidator
            {
                RequiredLength = 6,
                RequireNonLetterOrDigit = true,
                RequireDigit = true,
                RequireLowercase = true,
                RequireUppercase = true,
            };
            // 2 要素認証プロバイダーを登録します。このアプリケーションでは、Phone and Emails をユーザー検証用コード受け取りのステップとして使用します。
            // 2 要素認証の使用方法の詳細については、http://go.microsoft.com/fwlink/?LinkID=391935 を参照してください
            // 独自のプロバイダーを作成して、ここにプラグインすることができます。
            manager.RegisterTwoFactorProvider("PhoneCode", new PhoneNumberTokenProvider<ApplicationUser>
            {
                MessageFormat = "Your security code is: {0}"
            });
            manager.RegisterTwoFactorProvider("EmailCode", new EmailTokenProvider<ApplicationUser>
            {
                Subject = "SecurityCode",
                BodyFormat = "Your security code is: {0}"
            });
            manager.EmailService = new EmailService();
            manager.SmsService = new SmsService();
            var dataProtectionProvider = options.DataProtectionProvider;
            if (dataProtectionProvider != null)
            {
                manager.UserTokenProvider = new DataProtectorTokenProvider<ApplicationUser>(dataProtectionProvider.Create("ASP.NET Identity"));
            }

            //(ISY05)デフォルトロックセッティング追加
            manager.DefaultAccountLockoutTimeSpan = TimeSpan.FromMinutes(1);//テスト用に1分だけ
            manager.MaxFailedAccessAttemptsBeforeLockout = 4; //間違いは3回まで

            return manager;
        }

        /// <summary>
        /// (ISY04)ロールマネージャの追加 
        /// </summary>
        private RoleManager<IdentityRole> roleManager = new RoleManager<IdentityRole>(
               new RoleStore<IdentityRole>(new ApplicationDbContext()));

        /// <summary>
        /// (ISY04) Roleのリストを返す
        /// </summary>
        public List<IdentityRole> Roles
        {
            get { return roleManager.Roles.ToList(); }
        }

        /// <summary>
        /// (ISY04) Roleの名前での検索
        /// </summary>
        /// <param name="Name"></param>
        /// <returns></returns>
        public IdentityRole RoleByName(string Name)
        {
            return roleManager.FindByName(Name);
        }

        /// <summary>
        /// (ISY04) 新規Roleの作成
        /// </summary>
        /// <param name="name"></param>
        /// <param name="description"></param>
        /// <returns></returns>
        public bool CreateRole(string name, string description = "")
        {
            var role = this.RoleByName(name);
            if (role != null)
            {
                return false;
            }

            var idResult = roleManager.Create(new IdentityRole(name));

            return idResult.Succeeded;
        }
    }

    public class EmailService : IIdentityMessageService
    {
        public Task SendAsync(IdentityMessage message)
        {
            // 電子メールを送信するには、電子メール サービスをここにプラグインします。
            // (ISY01)ユーザ登録メール送信
            System.Net.Mail.MailMessage msg = new System.Net.Mail.MailMessage(
              "myaccount@gmail.com",
              message.Destination,
              message.Subject,
              message.Body);

            System.Net.Mail.SmtpClient sc = new System.Net.Mail.SmtpClient();

            //SMTPサーバーを設定する
            sc.Host = "smtp.gmail.com";
            sc.Port = 587;
            sc.DeliveryMethod = System.Net.Mail.SmtpDeliveryMethod.Network;

            //ユーザー名とパスワードを設定する
            sc.Credentials = new System.Net.NetworkCredential("myaccount@gmail.com", "mypassword");

            //SSLを使用する
            sc.EnableSsl = true;

            //メッセージを送信する
            sc.Send(msg);

            //後処理
            msg.Dispose();
            sc.Dispose();

            return Task.FromResult(0);
        }
    }

    public class SmsService : IIdentityMessageService
    {
        public Task SendAsync(IdentityMessage message)
        {
            // テキスト メッセージを送信するには、SMS サービスをここにプラグインします。
            return Task.FromResult(0);
        }
    }
}